@@ -57,6 +57,8 @@ static void  gdl_dock_bar_destroy         (GtkObject       *object);
 
 static void  gdl_dock_bar_attach          (GdlDockBar      *dockbar,
                                            GdlDockMaster   *master);
+static void gdl_dock_bar_remove_item      (GdlDockBar      *dockbar,
+                                           GdlDockItem     *item);
 
 /* ----- Class variables and definitions ----- */
 
@@ -176,6 +178,13 @@ gdl_dock_bar_set_property (GObject         *object,
     };
 }
 
+static void
+on_dock_item_foreach_disconnect (GdlDockItem *item, GdlDockBar *dock_bar)
+{
+    g_signal_handlers_disconnect_by_func (item, gdl_dock_bar_remove_item,
+                                          dock_bar);
+}
+
 static void
 gdl_dock_bar_destroy (GtkObject *object)
 {
@@ -183,6 +192,13 @@ gdl_dock_bar_destroy (GtkObject *object)
 
     if (dockbar->_priv) {
         GdlDockBarPrivate *priv = dockbar->_priv;
+
+        if (priv->items) {
+            g_slist_foreach (priv->items,
+                             (GFunc) on_dock_item_foreach_disconnect,
+                             object);
+            g_slist_free (priv->items);
+        }
         
         if (priv->master) {
             g_signal_handlers_disconnect_matched (priv->master,
@@ -228,6 +244,9 @@ gdl_dock_bar_remove_item (GdlDockBar  *dockbar,
     g_assert (button != NULL);
     gtk_container_remove (GTK_CONTAINER (dockbar), button);
     g_object_set_data (G_OBJECT (item), "GdlDockBarButton", NULL);
+    g_signal_handlers_disconnect_by_func (item,
+                                          G_CALLBACK (gdl_dock_bar_remove_item),
+                                          dockbar);
 }
 
 static void
@@ -282,7 +301,7 @@ gdl_dock_bar_add_item (GdlDockBar  *dockbar,
     else
         box = gtk_vbox_new (FALSE, 0);
     
-    g_object_get (item, "stock_id", &stock_id, "long_name", &name, NULL);
+    g_object_get (item, "stock-id", &stock_id, "long-name", &name, NULL);
 
     if (dockbar->_priv->dockbar_style == GDL_DOCK_BAR_TEXT ||
         dockbar->_priv->dockbar_style == GDL_DOCK_BAR_BOTH) {
@@ -320,6 +339,11 @@ gdl_dock_bar_add_item (GdlDockBar  *dockbar,
                       G_CALLBACK (gdl_dock_bar_item_clicked), item);
 
     gtk_widget_show_all (button);
+    
+    /* Set up destroy notify */
+    g_signal_connect_swapped (item, "destroy",
+                              G_CALLBACK (gdl_dock_bar_remove_item),
+                              dockbar);
 }
 
 static void
@@ -395,7 +419,7 @@ gdl_dock_bar_attach (GdlDockBar    *dockbar,
     dockbar->_priv->master = master;
     if (dockbar->_priv->master) {
         g_object_ref (dockbar->_priv->master);
-        g_signal_connect (dockbar->_priv->master, "layout_changed",
+        g_signal_connect (dockbar->_priv->master, "layout-changed",
                           G_CALLBACK (gdl_dock_bar_layout_changed_cb),
                           dockbar);
     }
@@ -933,3 +957,16 @@ void gdl_dock_bar_set_orientation (GdlDockBar *dockbar,
 
     gtk_widget_queue_resize (GTK_WIDGET (dockbar));
 }
+
+void gdl_dock_bar_set_style(GdlDockBar* dockbar,
+			    GdlDockBarStyle style)
+{
+    g_object_set(G_OBJECT(dockbar), "dockbar-style", style, NULL);
+}
+
+GdlDockBarStyle gdl_dock_bar_get_style(GdlDockBar* dockbar)
+{
+    GdlDockBarStyle style;
+    g_object_get(G_OBJECT(dockbar), "dockbar-style", &style, NULL);
+    return style;
+}
