@@ -3,21 +3,21 @@
  * This file is part of the GNOME Devtools Libraries.
  *
  * Copyright (C) 2002 Gustavo Giráldez <gustavo.giraldez@gmx.net>
+ *               2007 Naba Kumar  <naba@gnome.org>
+ * 
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.  
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -27,7 +27,6 @@
 #include "gdl-i18n.h"
 #include <stdlib.h>
 #include <string.h>
-#include <gdk/gdkx.h>
 
 #include "gdl-tools.h"
 #include "gdl-dock.h"
@@ -38,6 +37,9 @@
 
 #include "libgdlmarshal.h"
 
+#ifndef __FUNCTION__
+#define __FUNCTION__ __func__
+#endif
 
 /* ----- Private prototypes ----- */
 
@@ -181,7 +183,7 @@ gdl_dock_class_init (GdlDockClass *klass)
     
     g_object_class_install_property (
         g_object_class, PROP_DEFAULT_TITLE,
-        g_param_spec_string ("default_title", _("Default title"),
+        g_param_spec_string ("default-title", _("Default title"),
                              _("Default title for the newly created floating docks"),
                              NULL,
                              G_PARAM_READWRITE));
@@ -245,7 +247,7 @@ gdl_dock_class_init (GdlDockClass *klass)
     /* signals */
 
     dock_signals [LAYOUT_CHANGED] = 
-        g_signal_new ("layout_changed", 
+        g_signal_new ("layout-changed", 
                       G_TYPE_FROM_CLASS (klass),
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (GdlDockClass, layout_changed),
@@ -342,7 +344,7 @@ gdl_dock_constructor (GType                  type,
             /* set the title and connect to the long_name notify queue
                so we can reset the title when this prop changes */
             gdl_dock_set_title (dock);
-            g_signal_connect (dock, "notify::long_name",
+            g_signal_connect (dock, "notify::long-name",
                               (GCallback) gdl_dock_notify_cb, NULL);
             
             /* set transient for the first dock if that is a non-floating dock */
@@ -387,7 +389,7 @@ gdl_dock_set_property  (GObject      *object,
         case PROP_DEFAULT_TITLE:
             if (GDL_DOCK_OBJECT (object)->master)
                 g_object_set (GDL_DOCK_OBJECT (object)->master,
-                              "default_title", g_value_get_string (value),
+                              "default-title", g_value_get_string (value),
                               NULL);
             break;
         case PROP_WIDTH:
@@ -437,7 +439,7 @@ gdl_dock_get_property  (GObject      *object,
             if (GDL_DOCK_OBJECT (object)->master) {
                 gchar *default_title;
                 g_object_get (GDL_DOCK_OBJECT (object)->master,
-                              "default_title", &default_title,
+                              "default-title", &default_title,
                               NULL);
 #if GLIB_CHECK_VERSION(2,3,0)
                 g_value_take_string (value, default_title);
@@ -480,12 +482,12 @@ gdl_dock_set_title (GdlDock *dock)
         title = object->long_name;
     }
     else if (object->master) {
-        g_object_get (object->master, "default_title", &title, NULL);
+        g_object_get (object->master, "default-title", &title, NULL);
         free_title = TRUE;
     }
 
     if (!title && dock->root) {
-        g_object_get (dock->root, "long_name", &title, NULL);
+        g_object_get (dock->root, "long-name", &title, NULL);
         free_title = TRUE;
     }
     
@@ -1056,6 +1058,153 @@ gdl_dock_new_from (GdlDock  *original,
     return GTK_WIDGET (new_dock);
 }
 
+/* Depending on where the dock item (where new item will be docked) locates
+ * in the dock, we might need to change the docking placement. If the
+ * item is does not touches the center of dock, the new-item-to-dock would
+ * require a center dock on this item.
+ */
+static GdlDockPlacement
+gdl_dock_refine_placement (GdlDock *dock, GdlDockItem *dock_item,
+                           GdlDockPlacement placement)
+{
+    GtkRequisition object_size;
+    
+    gdl_dock_item_preferred_size (dock_item, &object_size);
+    g_return_val_if_fail (GTK_WIDGET (dock)->allocation.width > 0, placement);
+    g_return_val_if_fail (GTK_WIDGET (dock)->allocation.height > 0, placement);
+    g_return_val_if_fail (object_size.width > 0, placement);
+    g_return_val_if_fail (object_size.height > 0, placement);
+
+    if (placement == GDL_DOCK_LEFT || placement == GDL_DOCK_RIGHT) {
+        /* Check if dock_object touches center in terms of width */
+        if (GTK_WIDGET (dock)->allocation.width/2 > object_size.width) {
+            return GDL_DOCK_CENTER;
+        }
+    } else if (placement == GDL_DOCK_TOP || placement == GDL_DOCK_BOTTOM) {
+        /* Check if dock_object touches center in terms of height */
+        if (GTK_WIDGET (dock)->allocation.height/2 > object_size.height) {
+            return GDL_DOCK_CENTER;
+        }
+    }
+    return placement;
+}
+
+/* Determines the larger item of the two based on the placement:
+ * for left/right placement, height determines it.
+ * for top/bottom placement, width determines it.
+ * for center placement, area determines it.
+ */
+static GdlDockItem*
+gdl_dock_select_larger_item (GdlDockItem *dock_item_1,
+                             GdlDockItem *dock_item_2,
+                             GdlDockPlacement placement,
+                             gint level /* for debugging */)
+{
+    GtkRequisition size_1, size_2;
+    
+    g_return_val_if_fail (dock_item_1 != NULL, dock_item_2);
+    g_return_val_if_fail (dock_item_2 != NULL, dock_item_1);
+    
+    gdl_dock_item_preferred_size (dock_item_1, &size_1);
+    gdl_dock_item_preferred_size (dock_item_2, &size_2);
+    
+    g_return_val_if_fail (size_1.width > 0, dock_item_2);
+    g_return_val_if_fail (size_1.height > 0, dock_item_2);
+    g_return_val_if_fail (size_2.width > 0, dock_item_1);
+    g_return_val_if_fail (size_2.height > 0, dock_item_1);
+    
+    if (placement == GDL_DOCK_LEFT || placement == GDL_DOCK_RIGHT)
+    {
+        /* For left/right placement, height is what matters */
+        return (size_1.height >= size_2.height?
+                    dock_item_1 : dock_item_2);
+    } else if (placement == GDL_DOCK_TOP || placement == GDL_DOCK_BOTTOM)
+    {
+        /* For top/bottom placement, width is what matters */
+        return (size_1.width >= size_2.width?
+                    dock_item_1 : dock_item_2);
+    } else if (placement == GDL_DOCK_CENTER) {
+        /* For center place, area is what matters */
+        return ((size_1.width * size_1.height)
+                    >= (size_2.width * size_2.height)?
+                    dock_item_1 : dock_item_2);
+    } else {
+        g_warning ("Should not reach here: %s:%d", __FUNCTION__, __LINE__);
+    }
+    return dock_item_1;
+}
+
+/* Determines the best dock item to dock a new item with the given placement.
+ * It traverses the dock tree and (based on the placement) tries to find
+ * the best located item wrt to the placement. The approach is to find the
+ * largest item on/around the placement side (for side placements) and to
+ * find the largest item for center placement. In most situations, this is
+ * what user wants and the heuristic should be therefore sufficient.
+ */
+static GdlDockItem*
+gdl_dock_find_best_placement_item (GdlDockItem *dock_item,
+                                   GdlDockPlacement placement,
+                                   gint level /* for debugging */)
+{
+    GdlDockItem *ret_item = NULL;
+    
+    if (GDL_IS_DOCK_PANED (dock_item))
+    {
+        GtkOrientation orientation;
+        GdlDockItem *dock_item_1, *dock_item_2;
+        GList* children;
+        
+        children = gtk_container_get_children (GTK_CONTAINER (dock_item));
+        
+        g_assert (g_list_length (children) == 2);
+        
+        g_object_get (dock_item, "orientation", &orientation, NULL);
+        if ((orientation == GTK_ORIENTATION_HORIZONTAL &&
+             placement == GDL_DOCK_LEFT) ||
+            (orientation == GTK_ORIENTATION_VERTICAL &&
+             placement == GDL_DOCK_TOP)) {
+            /* Return left or top pane widget */
+            ret_item =
+                gdl_dock_find_best_placement_item (GDL_DOCK_ITEM
+                                                   (children->data),
+                                                   placement, level + 1);
+        } else if ((orientation == GTK_ORIENTATION_HORIZONTAL &&
+                    placement == GDL_DOCK_RIGHT) ||
+                   (orientation == GTK_ORIENTATION_VERTICAL &&
+                    placement == GDL_DOCK_BOTTOM)) {
+                        /* Return right or top pane widget */
+            ret_item =
+                gdl_dock_find_best_placement_item (GDL_DOCK_ITEM
+                                                   (children->next->data),
+                                                   placement, level + 1);
+        } else {
+            /* Evaluate which of the two sides is bigger */
+            dock_item_1 =
+                gdl_dock_find_best_placement_item (GDL_DOCK_ITEM
+                                                   (children->data),
+                                                   placement, level + 1);
+            dock_item_2 =
+                gdl_dock_find_best_placement_item (GDL_DOCK_ITEM
+                                                   (children->next->data),
+                                                   placement, level + 1);
+            ret_item = gdl_dock_select_larger_item (dock_item_1,
+                                                    dock_item_2,
+                                                    placement, level);
+        }
+        g_list_free (children);
+    }
+    else if (GDL_IS_DOCK_ITEM (dock_item))
+    {
+        ret_item = dock_item;
+    }
+    else
+    {
+        /* should not be here */
+        g_warning ("Should not reach here: %s:%d", __FUNCTION__, __LINE__);
+    }
+    return ret_item;
+}
+
 void
 gdl_dock_add_item (GdlDock          *dock,
                    GdlDockItem      *item,
@@ -1069,10 +1218,25 @@ gdl_dock_add_item (GdlDock          *dock,
         gdl_dock_add_floating_item (dock, item, 0, 0, -1, -1);
 
     else {
+        GdlDockItem *best_dock_item;
         /* Non-floating item. */
-        gdl_dock_object_dock (GDL_DOCK_OBJECT (dock),
-                              GDL_DOCK_OBJECT (item),
-                              placement, NULL);
+        if (dock->root) {
+            GdlDockPlacement local_placement;
+            GtkRequisition preferred_size;
+            
+            best_dock_item =
+                gdl_dock_find_best_placement_item (GDL_DOCK_ITEM (dock->root),
+                                                   placement, 0);
+            local_placement = gdl_dock_refine_placement (dock, best_dock_item,
+                                                         placement);
+            gdl_dock_object_dock (GDL_DOCK_OBJECT (best_dock_item),
+                                  GDL_DOCK_OBJECT (item),
+                                  local_placement, NULL);
+        } else {
+            gdl_dock_object_dock (GDL_DOCK_OBJECT (dock),
+                                  GDL_DOCK_OBJECT (item),
+                                  placement, NULL);
+        }
     }
 }
 
