@@ -3,20 +3,19 @@
  *
  * Copyright (C) 2000-2001 Dave Camp
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  * Authors: Dave Camp, Jeroen Zwartepoorte
  */
@@ -108,7 +107,8 @@ gdl_icons_dispose (GObject *object)
 	GdlIconsPrivate *priv = GDL_ICONS_GET_PRIVATE (object);
 
 	if (priv->icon_theme) {
-		g_object_unref (priv->icon_theme);
+		/* Don't do that - look a GTK+ docs */
+		/* g_object_unref (priv->icon_theme); */
 		priv->icon_theme = NULL;
 	}
 	
@@ -146,7 +146,8 @@ gdl_icons_instance_init (GdlIcons *icons)
 
 	priv->icon_theme = gtk_icon_theme_get_default ();
 	/* gtk_icon_theme_get_default() does not ref the returned object */
-	g_object_ref (priv->icon_theme);
+	/* but API docs state the you should NOT ref it */
+	/* g_object_ref (priv->icon_theme);*/
 	g_signal_connect_object (G_OBJECT (priv->icon_theme), "changed",
 				 G_CALLBACK (theme_changed_cb), icons, 0);
 	priv->icons = g_hash_table_new_full (g_str_hash, g_str_equal,
